% Fig5-31 & 5-32
% Externally Modulated 10Gb/s Single-channel Link with APD

clear all ;% clear all variables
close all
%============== Define global variables ===================================================
global BitRate Bits2Power Bits ChannelNumber ChannelSpacing TimeWindow SampleRate DataLength samplesperbit t_range f_sym Ref_Frequency

BitRate = 10e9;  
Bits2Power = 10;
Bits = 2^Bits2Power;
TimeWindow = Bits/BitRate;
ChannelNumber = 1;
ChannelSpacing = 100e9;   %[Hz]

SampleRateMargin = 5;  %MarginBandwidth = SampleRateMargin*ChannelSpacing
SampleRate = BitRate*SampleRateFactor(SampleRateMargin); %[Hz]
DataLength = TimeWindow*SampleRate;
Ref_Frequency = 193.41e12; %[Hz] Reference optical frequency, usually center frequency of simulations

samplesperbit = SampleRate/BitRate; % data number per bit

% time, optical frequency scale
t_range = 0:1/SampleRate:1/SampleRate*(DataLength-1);
f_sym = -.5/TimeWindow*(DataLength):1/TimeWindow:.5/TimeWindow*(DataLength-1);

%==========================================================================================
%=================== Define Simulation Blocks =============================================
%-------------- Laser  -----------------------%
LaserNumber = ChannelNumber;
LaserSpacing = ChannelSpacing; %[Hz] Equal channel spacing
UnmodulatedPower = 2e-3; %[W] %Fixed. 
Linewidth = 10e6; %[Hz]
Laser1 = LaserCW(UnmodulatedPower,Linewidth);
%------------- Data Source -------------------%
PreSpaces = 1;  %number of leading zero bits
PostSpaces = 1; %number of trailing zero bits
%PRBS_type = 'PRBS_Fixed'; %{'PRBS','PRBS_Fixed','Alternate','Ones','Zeros','File'}
%PRBS_File = 'bits_64.dat';%, will be used only if PRBS_type = 'File'
info_bits = infobits_PRBS(PreSpaces,PostSpaces);
S_data = NRZCoder(info_bits);

RiseTime = 1/4/BitRate; %  In the form of k/BitRate, k should be less than (1/2)*RZRatio   

S_elec = RiseTimeAdjust(S_data,RiseTime);

%----------- Modulation -----------------------%
ModulationIndex = 0.98;
Mod_y = ModulatorAmplitude(Laser1,S_elec,ModulationIndex);
 
%==================================================================================================
%--------- 1.SMF Fiber --------------------%
%=== Physical Parameters =========
S_FiberAttenuation = 0.2e-3; %[dB/m]
S_Fiber_Length = 50e3; %[m]
S_Dispersion = 16e-6;   %[s/m^2]
S_DispersionSlope = 0.08e3;   %[s/m^3]
S_NonlinearIndex = 2.6e-20; %[m^2/W]
S_CoreArea = 80e-12; %[m^2]
S_MaxPhaseChange = 0.001; %[rad]

SMF_out = SingleModeFiber(Mod_y,S_Fiber_Length,S_FiberAttenuation,S_Dispersion,...
                               S_DispersionSlope,S_NonlinearIndex,S_CoreArea,S_MaxPhaseChange);
                      
%--------- Received Power Control ----------------------- 
atten = 0; %[dB]
attenuated_out = Attenuator(SMF_out,atten);
ref_signal = Attenuator(Mod_y,S_FiberAttenuation*S_Fiber_Length);

%============= Rx APD =====================================
%===Physical Parameters=======
Responsivity = 1; %[A/W]
ThermalNoiseDen = 1e-20; %[A/sqrt(Hz)]
DarkCurrMult = 1e-10; %[A]
DarkCurrNonMult = 0; %[A], [0.10e-6]
AvalacheMul = 3  ;
IonizationCoeff = .7;
ShotNoiseFactor =  1;   % = 1 if shot noise included, = 0 if not. 
FilterOrder = 3;
APD_BW = 0.75*BitRate; % Cutoff Frequency in [Hz]

i_APD1 = Rx_APD(attenuated_out,Responsivity,AvalacheMul,IonizationCoeff,ShotNoiseFactor,ThermalNoiseDen,...
                DarkCurrMult,DarkCurrNonMult,FilterOrder,APD_BW);
i_APD_ref = Rx_APD(ref_signal,Responsivity,AvalacheMul,IonizationCoeff,ShotNoiseFactor,ThermalNoiseDen,...
                DarkCurrMult,DarkCurrNonMult,FilterOrder,APD_BW);
           
EyePlotter(i_APD1)   
EyePlotter(i_APD_ref)

%======== EyeAnalyzer ====================================     
SampleTime = 0.5 ;

Out1 = EyeAnalyzer_Relative(i_APD1,info_bits,SampleTime);
Out2 = EyeAnalyzer_Relative(i_APD_ref,info_bits,SampleTime);

