clear all ;% clear all variables
close all

%============== Define global variables ===================================================
global BitRate Bits TimeWindow SampleRate DataLength samplesperbit t_range f_sym Ref_Frequency

BitRate = 1e9;  
Bits2Power = 8;
Bits = 2^Bits2Power;
TimeWindow = Bits/BitRate;
SampleRate = BitRate*2^10;
DataLength = TimeWindow*SampleRate;
Ref_Frequency = 193.1e12; %[Hz] Reference optical frequency
samplesperbit = SampleRate/BitRate; % data number per bit

% time, optical frequency range
t_range = 0:1/SampleRate:1/SampleRate*(DataLength-1);
f_sym = SampleRate*(-DataLength/2:(DataLength/2-1))/DataLength; %

%==========================================================================================
%=================== Define Simulation Blocks =============================================
% SM Laser
UnmodulatedPower = 2e-3; %[W] 
Linewidth = 10e6; %[Hz]
Laser1 = LaserCW(UnmodulatedPower,Linewidth);
%------------- Data Source -------------------%
PreSpaces = 2;  %number of leading zero bits
PostSpaces = 2; %number of trailing zero bits
info_bits = infobits_PRBS(PreSpaces,PostSpaces);

E_data = NRZCoder(info_bits);

ERiseTime = 1/4/BitRate; %  In the form of k/BitRate, k should be less than (1/2)*RZRatio if RZCoder used   
E_elec = RiseTimeAdjust(E_data,ERiseTime);

ER = 30;% Extinction Ratio [dB]
ModulationIndex = 1-10^(-ER/10); % Note)Extinction Ratio [dB] = -10log10(1-ModulationIndex)
mod_out = ModulatorAmplitude(Laser1,E_elec,ModulationIndex);

%---------------- Transmission -------------------------------------
loss = 25; % [dB]
%loss = input('What is the Attenuation in [dB]?  '); % 20 ~ 30 [dB]
mod_att = Attenuator(mod_out,loss);

%--------------- Receiver1 (PIN) ------------------------------------------
Responsivity = 1.0;% [A/W], [0,2]
ThermalNoiseDen = 1e-22;% [A^2/Hz] 
DarkCurrent = 0;% [A], [0,10e-6]
ShotNoiseFactor = 0;%   (1 if shot noise included, 0 if not)

y_PIN = PhotoDiode_PIN(mod_att,Responsivity,ThermalNoiseDen,DarkCurrent,ShotNoiseFactor); 

FilterOrder = 3;
BW3dB = 0.75*BitRate;

y_PIN_LP = E_FilterLPBessel(y_PIN,FilterOrder,BW3dB);
%--------------- Ouput Observe & BER Calculation --------------------------------------

EyePlotter(y_PIN_LP)
title('PIN Receiver Output')
BER_PIN = qnber_noISI(y_PIN_LP,info_bits,0.5, 0.5);

%--------------- Receiver2 (APD) ------------------------------------------
DarkCurrMult = 0 ;%[A], [0,10e-6]
DarkCurrNonMult = 0 ;% [A], [0.10e-6]
AvalacheMul = 10; %   [1,1000]
IonizationCoeff = 0.6;%  [1e-3, 1]

y_APD = PhotoDiode_APD(mod_att,Responsivity,AvalacheMul,IonizationCoeff,...
    ShotNoiseFactor,ThermalNoiseDen,DarkCurrMult,DarkCurrNonMult);
%        
y_APD_LP = E_FilterLPBessel(y_APD,FilterOrder,BW3dB);

%--------------- Ouput Observation & BER Calculation --------------------------------------

EyePlotter(y_APD_LP)
title('APD Receiver Output')
BER_APD = qnber_noISI(y_APD_LP,info_bits,0.5, 0.5);

%==========================================================================
%---- PIN Receiver LPF Bandwidth Control -----------------------------

BW3dBk = (0.2:0.1:2)*BitRate;

for k = 1:length(BW3dBk);
    
    y_PIN_LP = E_FilterLPBessel(y_PIN,FilterOrder,BW3dBk(k));
    BER_PIN(k) = qnber_noISI(y_PIN_LP,info_bits,0.5, 0.5);
    
end

figure(3)
semilogy(BW3dBk,BER_PIN,'-*')
xlabel('뿪 [Hz]'), ylabel('BER')






